﻿using GrapeCity.Forguncy.ServerApi;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DBImport
{
    public class DBImport : ForguncyApi
    {

        /// <summary>
        /// 毎日12時半にすべての内部データベースのデータを削除してデータを取り込みます。
        /// </summary>
        [ScheduleApi("12:30:00", "1")]
        public void RefreshAllTable_Schedule()
        {
            // Forguncy アプリ側で状態管理が必要な際には、ここで内部テーブルの状態を変更します。

            // 内部データベース（テーブル2）のデータを削除します。
            int retDel = DeleteInnerTable("テーブル2", "テーブル2");

            // 内部データベースへデータを取り込みます。（テーブル2にテーブル1をコピーします）
            int retCopy = CopyTbl("テーブル2", "テーブル1");
 
            // Forguncy アプリ側で状態管理が必要な際には、ここで内部テーブルの状態を戻します。
        }


        /// <summary>
        /// デーブル2（内部テーブル）へデータをコピーします。
        /// </summary>
        [Post]
        public void copy_table()
        {
            // 内部テーブルへ指定したデータをコピーします。（テーブル2にテーブル1をコピーします）
            // ※コピーに時間がかかる際には、第二引数でOData関数を利用して所得するフィールドを絞り込みます。
            int retCopy = CopyTbl("テーブル2", "テーブル1");
        }



        /// <summary>
        /// テーブル1のデータをコピーします。
        /// </summary>
        /// <param name="strTableName">データを保存するテーブル</param>
        /// <param name="strODataCondition">取り込むデータの抽出用OData文字列（テーブル名だとすべてのデータ）</param>
        /// <returns>コピーしたレコード数</returns>
        private int CopyTbl(string strTableName, string strODataCondition)
        {
            int intReturnl = 0;

            // GetTableData (string odataPath)メソッドのOData指定を利用して各レコードの情報を取得します。
            List<Dictionary<string, object>> dicRecords = new List<Dictionary<string, object>>();
            dicRecords = (List<Dictionary<string, object>>)this.DataAccess.GetTableData(strODataCondition);

            // コピーするデータを生成します。
            var data = new Dictionary<string, ModifyData>();
            var modifyData = new ModifyData();

            foreach (Dictionary<string, object> dicRecord in dicRecords)
            {
                // 1レコード分のデータを生成します。内部データベースへ取り込むフィールド情報を記載します。
                modifyData.AddRows.Add(new Dictionary<string, object>
                        {
                            {"NO", dicRecord["NO"] },
                            {"COUNTRY_CD", dicRecord["COUNTRY_CD"] },
                            {"ROOT", dicRecord["ROOT"] },
                            {"APP_KIND", dicRecord["APP_KIND"] },
                            {"APP_DATE", dicRecord["APP_DATE"] },
                            {"REG_DATE", dicRecord["REG_DATE"] }
                        });
                intReturnl += 1;
            }

            // テーブルの変更を反映します。
            data.Add(strTableName, modifyData);
            this.DataAccess.ModifyTablesData(data);

            return (intReturnl);
        }

  
        /// <summary>
        /// テーブル2（内部データベース）のデータを削除します。
        /// </summary>
        [Post]
        public void delete_table2()
        {
            // すべてのレコードを削除します。
            // ※取得に時間がかかる際には、第二引数でOData関数を利用してIDフィールドのみに絞り込みます。
            int intretval = DeleteInnerTable("テーブル2", "テーブル2");
        }

 
 
        /// <summary>
        /// API関数を利用してレコードを削除します。
        /// </summary>
        /// <param name="strTableName">テーブル名</param>
        /// <param name="strODataCondition">削除するレコードのODataでの抽出条件。テーブル名を指定すると全データが対象になります。</param>
        /// <returns></returns>
        private int DeleteInnerTable(string strTableName, string strODataCondition)
        {
            int intReturnl = 0;

            // GetTableData (string odataPath)メソッドのOData指定を利用して各レコードの情報を取得します。
            List<Dictionary<string, object>> dicRecords = new List<Dictionary<string, object>>();
            dicRecords = (List<Dictionary<string, object>>)this.DataAccess.GetTableData(strODataCondition);

            // 削除情報を生成します。
            var data = new Dictionary<string, ModifyData>();
            var modifyData = new ModifyData();

            foreach (Dictionary<string, object> dicRecord in dicRecords)
            {
                // フィールド情報を取り出します。
                Dictionary<string, object> dicFields = new Dictionary<string, object>();
                dicFields = (Dictionary<string, object>)dicRecord;

                // 削除情報を生成します。
                modifyData.DeleteRows.Add(new ColumnValuePair()
                {
                    ColumnName = "ID",
                    Value = dicRecord["ID"]
                });
                intReturnl += 1;
            }

            // テーブルの変更を反映（レコードを削除）します。
            data.Add(strTableName, modifyData);
            this.DataAccess.ModifyTablesData(data);

            return (intReturnl);

        }


    }
}
